package Role::Child;
# This is automatically generated by author/import-moose-test.pl.
# DO NOT EDIT THIS FILE. ANY CHANGES WILL BE LOST!!!
use t::lib::MooseCompat;
use Mouse::Role;

with 'Role::Parent' => { -alias => { meth1 => 'aliased_meth1', } };

sub meth1 { }

1;
